/* daytime.c */
/* Copyright INMOS Limited 1990,1991 */
/* Version 1.2 */

/*
 ******************************************************************************
 *
 * File:        daytime.c
 *
 * Purpose:     Implement 'daytime' command.
 *		Uses 'DAYTIME' protocol, as defined in RFC 867.
 *
 * Author:      Graham W. Hudspith
 *
 * Date:        7/4/91
 *
 ******************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <iocntrl.h>
#include <string.h>
#include <getopt.h>

#include <serrno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

int main (int argc, char **argv)
{
    int             udp, ch, stype, s, n;
    struct hostent *hp;
    struct sockaddr_in remote;
    char            date[80];

    extern void usage (void);

    udp = 0;

    while ((ch = getopt (argc, argv, "ut")) != EOF)
    {
	switch (ch)
	{
	case 't':
	    break;

	case 'u':
	    udp = 1;
	    break;

	case '?':
	default:
	    usage ();
	}

    }
    argc -= optind;
    argv += optind;

    if ((hp = gethostbyname ("cheetah")) == NULL)
    {
	sperror ("gethostbyname");
	exit (1);
    }

    (void) memset ((char *) &remote, '\0', sizeof (remote));
    remote.sin_family = PF_INET;
    remote.sin_port = htons (IPPORT_DAYTIME);
    (void) memcpy ((char *) &(remote.sin_addr),
		   hp->h_addr_list[0], hp->h_length);

    if (udp)
    {
	stype = SOCK_DGRAM;
    }
    else
    {
	stype = SOCK_STREAM;
    }

    if ((s = socket (AF_INET, stype, 0)) < 0)
    {
	sperror ("socket");
	exit (1);
    }

    if (connect (s, (struct sockaddr *) & remote, sizeof (remote)) < 0)
    {
	sperror ("connect");
	exit (1);
    }

    if (udp)
    {
	if (write (s, date, sizeof (date)) < sizeof (date))
	{
	    sperror ("write");
	    exit (1);
	}
	n = read (s, date, sizeof (date));
	if (n < 0)
	{
	    sperror ("read");
	    exit (1);
	}
	else if (n > 0)
	{
	    (void) write (1, date, n);
	}
    }
    else
    {
	while ((n = read (s, date, sizeof (date))) > 0)
	{
	    (void) write (1, date, n);
	}
    }

    (void) close (s);
    exit (0);
}

void
usage (void)
{
    (void) fprintf (stderr, "usage: daytime [ -ut ]\n");
    exit (1);
}
