/* ip_icmp.h */
/* Copyright INMOS Limited 1990,1991,1992 */
/* Version 1.19 */

#ifndef __NETINET_IP_ICMP_H
#define __NETINET_IP_ICMP_H

/*
 * Copyright (c) 1980,1983,1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of California at Berkeley. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

/*
 * Interface Control Message Protocol Definitions.
 * Per RFC 792, September 1981.
 */

/*
 * Structure of an icmp header.
 */
struct icmp
{
    unsigned int        icmp_type:8,    /* type of message, see below */
                        icmp_code:8,    /* type sub code */
                        icmp_cksum:16;  /* ones complement cksum of struct */
    union
    {
        unsigned int    ih_pptr:8;      /* ICMP_PARAMPROB */
        struct in_addr  ih_gwaddr;      /* ICMP_REDIRECT */
        struct ih_idseq
        {
            unsigned int  icd_id:16,
                          icd_seq:16;
        }               ih_idseq;
        unsigned int    ih_void:32;
    }               icmp_hun;
#define icmp_pptr       icmp_hun.ih_pptr
#define icmp_gwaddr     icmp_hun.ih_gwaddr
#define icmp_id         icmp_hun.ih_idseq.icd_id
#define icmp_seq        icmp_hun.ih_idseq.icd_seq
#define icmp_void       icmp_hun.ih_void
    union
    {
        struct id_ts
        {
            unsigned long   its_otime;
            unsigned long   its_rtime;
            unsigned long   its_ttime;
        }               id_ts;
        struct id_ip
        {
            struct ip       idi_ip;
            /* options and then 64 bits of data */
        }               id_ip;
        unsigned long   id_mask;
        char            id_data[1];
    }               icmp_dun;
#define icmp_otime      icmp_dun.id_ts.its_otime
#define icmp_rtime      icmp_dun.id_ts.its_rtime
#define icmp_ttime      icmp_dun.id_ts.its_ttime
#define icmp_ip         icmp_dun.id_ip.idi_ip
#define icmp_mask       icmp_dun.id_mask
#define icmp_data       icmp_dun.id_data
};

/*
 * Lower bounds on packet lengths for various types.
 * For the error advice packets must first insure that the
 * packet is large enought to contain the returned ip header.
 * Only then can we do the check to see if 64 bits of packet
 * data have been returned, since we need to check the returned
 * ip header length.
 */
#define ICMP_MINLEN     8       /* abs minimum */
#define ICMP_TSLEN      (8 + 3 * sizeof (unsigned long))        /* timestamp */
#define ICMP_MASKLEN    12      /* address mask */
#define ICMP_ADVLENMIN  (8 + sizeof (struct ip) + 8)    /* min */
#define ICMP_ADVLEN(p)  (8 + ((p)->icmp_ip.ip_hl << 2) + 8)
 /* N.B.: must separately check that ip_hl >= 5 */

/*
 * Definition of type and code field values.
 */
#define ICMP_ECHOREPLY          0       /* echo reply */
#define ICMP_UNREACH            3       /* dest unreachable, codes: */
#define         ICMP_UNREACH_NET        0       /* bad net */
#define         ICMP_UNREACH_HOST       1       /* bad host */
#define         ICMP_UNREACH_PROTOCOL   2       /* bad protocol */
#define         ICMP_UNREACH_PORT       3       /* bad port */
#define         ICMP_UNREACH_NEEDFRAG   4       /* IP_DF caused drop */
#define         ICMP_UNREACH_SRCFAIL    5       /* src route failed */
#define ICMP_SOURCEQUENCH       4       /* packet lost, slow down */
#define ICMP_REDIRECT           5       /* shorter route, codes: */
#define         ICMP_REDIRECT_NET       0       /* for network */
#define         ICMP_REDIRECT_HOST      1       /* for host */
#define         ICMP_REDIRECT_TOSNET    2       /* for tos and net */
#define         ICMP_REDIRECT_TOSHOST   3       /* for tos and host */
#define ICMP_ECHO               8       /* echo service */
#define ICMP_TIMXCEED           11      /* time exceeded, code: */
#define         ICMP_TIMXCEED_INTRANS   0       /* ttl==0 in transit */
#define         ICMP_TIMXCEED_REASS     1       /* ttl==0 in reass */
#define ICMP_PARAMPROB          12      /* ip header bad */
#define ICMP_TSTAMP             13      /* timestamp request */
#define ICMP_TSTAMPREPLY        14      /* timestamp reply */
#define ICMP_IREQ               15      /* information request */
#define ICMP_IREQREPLY          16      /* information reply */
#define ICMP_MASKREQ            17      /* address mask request */
#define ICMP_MASKREPLY          18      /* address mask reply */

#define ICMP_MAXTYPE            18

#endif
