-- socket.inc
-- Copyright INMOS Limited 1990,1991,1992
-- Version 1.15

--
-- This header file is based in part on the Fourth Berkeley Software
-- Distribution. We acknowledge the Regents of the University of
-- California and the Electrical Engineering and Computer Science
-- Department at the Berkeley Campus of the University of California
-- and other contributors for their roles in its development.
--

--
-- Definitions related to sockets: types, address families, options.
--

--
-- Types
--
VAL  soc.SOCK.STREAM       IS 1:    -- stream socket
VAL  soc.SOCK.DGRAM        IS 2:    -- datagram socket
VAL  soc.SOCK.RAW          IS 3:    -- raw-protocol interface
VAL  soc.SOCK.RDM          IS 4:    -- reliably-delivered message 
VAL  soc.SOCK.SEQPACKET    IS 5:    -- sequenced packet stream

--
-- Option flags per-socket.
--
VAL  soc.SO.DEBUG          IS #0001:    -- turn on debugging info recording
VAL  soc.SO.ACCEPTCONN     IS #0002:    -- socket has had listen()
VAL  soc.SO.REUSEADDR      IS #0004:    -- allow local address reuse
VAL  soc.SO.KEEPALIVE      IS #0008:    -- keep connections alive
VAL  soc.SO.DONTROUTE      IS #0010:    -- just use interface addresses
VAL  soc.SO.BROADCAST      IS #0020:    -- permit sending of broadcast msgs
VAL  soc.SO.USELOOPBACK    IS #0040:    -- bypass hardware when possible
VAL  soc.SO.LINGER         IS #0080:    -- linger on close if data present
VAL  soc.SO.OOBINLINE      IS #0100:    -- leave received OOB data in line

--
-- Additional options, not kept in so.options.
--
VAL  soc.SO.SNDBUF         IS #1001:    -- send buffer size 
VAL  soc.SO.RCVBUF         IS #1002:    -- receive buffer size 
VAL  soc.SO.SNDLOWAT       IS #1003:    -- send low-water mark
VAL  soc.SO.RCVLOWAT       IS #1004:    -- receive low-water mark
VAL  soc.SO.SNDTIMEO       IS #1005:    -- send timeout
VAL  soc.SO.RCVTIMEO       IS #1006:    -- receive timeout
VAL  soc.SO.ERROR          IS #1007:    -- get error status and clear
VAL  soc.SO.TYPE           IS #1008:    -- get socket type

--
-- INMOS-specific options.
--
VAL  soc.IMS.LENGTHS       IS #8001:    -- retrieve buffer-lengths info

--
-- Level number for (get/set)sockopt() to apply to socket itself.
--
VAL  soc.SOL.SOCKET        IS #FFFF:    -- options for socket level 

--
-- Address families.
--
VAL  soc.AF.UNSPEC         IS 0:    -- unspecified
VAL  soc.AF.UNIX           IS 1:    -- local to host (pipes, portals)
VAL  soc.AF.INET           IS 2:    -- internetwork: UDP, TCP, etc.
VAL  soc.AF.IMPLINK        IS 3:    -- arpanet imp addresses
VAL  soc.AF.PUP            IS 4:    -- pup protocols: e.g. BSP
VAL  soc.AF.CHAOS          IS 5:    -- mit CHAOS protocols
VAL  soc.AF.NS             IS 6:    -- XEROX NS protocols
VAL  soc.AF.NBS            IS 7:    -- nbs protocols
VAL  soc.AF.ECMA           IS 8:    -- european computer manufacturers
VAL  soc.AF.DATAKIT        IS 9:    -- datakit protocols
VAL  soc.AF.CCITT          IS 10:    -- CCITT protocols, X.25 etc
VAL  soc.AF.SNA            IS 11:    -- IBM SNA
VAL  soc.AF.DECnet         IS 12:    -- DECnet
VAL  soc.AF.DLI            IS 13:    -- Direct data link interface
VAL  soc.AF.LAT            IS 14:    -- LAT
VAL  soc.AF.HYLINK         IS 15:    -- NSC Hyperchannel
VAL  soc.AF.APPLETALK      IS 16:    -- Apple Talk

VAL  soc.AF.NIT            IS 17:    -- Network Interface Tap
VAL  soc.AF.802            IS 18:    -- IEEE 802.2, also ISO 8802
VAL  soc.AF.OSI            IS 19:    -- umbrella for all families used by OSI (e.g.
                                   -- protosw lookup)
VAL  soc.AF.X25            IS 20:    -- CCITT X.25 in particular
VAL  soc.AF.OSINET         IS 21:    -- AFI = 47, IDI = 4
VAL  soc.AF.GOSIP          IS 22:    -- U.S. Government OSI

VAL  soc.AF.MAX            IS 23:

--
-- Protocol families, same as address families for now.
--
VAL  soc.PF.UNSPEC         IS soc.AF.UNSPEC:
VAL  soc.PF.UNIX           IS soc.AF.UNIX:
VAL  soc.PF.INET           IS soc.AF.INET:
VAL  soc.PF.IMPLINK        IS soc.AF.IMPLINK:
VAL  soc.PF.PUP            IS soc.AF.PUP:
VAL  soc.PF.CHAOS          IS soc.AF.CHAOS:
VAL  soc.PF.NS             IS soc.AF.NS:
VAL  soc.PF.NBS            IS soc.AF.NBS:
VAL  soc.PF.ECMA           IS soc.AF.ECMA:
VAL  soc.PF.DATAKIT        IS soc.AF.DATAKIT:
VAL  soc.PF.CCITT          IS soc.AF.CCITT:
VAL  soc.PF.SNA            IS soc.AF.SNA:
VAL  soc.PF.DECnet         IS soc.AF.DECnet:
VAL  soc.PF.DLI            IS soc.AF.DLI:
VAL  soc.PF.LAT            IS soc.AF.LAT:
VAL  soc.PF.HYLINK         IS soc.AF.HYLINK:
VAL  soc.PF.APPLETALK      IS soc.AF.APPLETALK:
VAL  soc.PF.NIT            IS soc.AF.NIT:
VAL  soc.PF.802            IS soc.AF.802:
VAL  soc.PF.OSI            IS soc.AF.OSI:
VAL  soc.PF.X25            IS soc.AF.X25:
VAL  soc.PF.OSINET         IS soc.AF.OSINET:
VAL  soc.PF.GOSIP          IS soc.AF.GOSIP:

VAL  soc.PF.MAX            IS soc.AF.MAX:

--
-- Maximum queue length specifiable by listen.
--
VAL  soc.SOMAXCONN         IS 5:

VAL  soc.MSG.OOB           IS #1:    -- process out-of-band data
VAL  soc.MSG.PEEK          IS #2:    -- peek at incoming message
VAL  soc.MSG.DONTROUTE     IS #4:    -- send without using routing tables

VAL  soc.MSG.MAXIOVLEN     IS 16:

---------------------------------------------------------------------------

--
-- maximum sizes of various args to socket library calls.
--

--
-- max size of a socket identifier array passed to soc.select ().
--
VAL  soc.MAX.SELECTIDS     IS 64:

--
-- sockaddr = INT16 + INT16 + INT32 + [8]BYTE
--
VAL  soc.SOCKADDR.LEN      IS 16:

---------------------------------------------------------------------------

--
-- Protocols
--
VAL  soc.IPPROTO.IP        IS 0:     -- dummy for IP
VAL  soc.IPPROTO.ICMP      IS 1:     -- control message protocol
VAL  soc.IPPROTO.IGMP      IS 2:     -- group control protocol
VAL  soc.IPPROTO.GGP       IS 3:     -- gateway^2 (deprecated)
VAL  soc.IPPROTO.TCP       IS 6:     -- tcp
VAL  soc.IPPROTO.EGP       IS 8:     -- exterior gateway protocol
VAL  soc.IPPROTO.PUP       IS 12:    -- pup
VAL  soc.IPPROTO.UDP       IS 17:    -- user datagram protocol
VAL  soc.IPPROTO.IDP       IS 22:    -- xns idp
VAL  soc.IPPROTO.ND        IS 77:    -- UNOFFICIAL net disk proto

VAL  soc.IPPROTO.RAW       IS 255:   -- raw IP packet
VAL  soc.IPPROTO.MAX       IS 256:

--
-- Port/socket numbers: network standard functions
--
VAL  soc.IPPORT.ECHO       IS 7:
VAL  soc.IPPORT.DISCARD    IS 9:
VAL  soc.IPPORT.SYSTAT     IS 11:
VAL  soc.IPPORT.DAYTIME    IS 13:
VAL  soc.IPPORT.NETSTAT    IS 15:
VAL  soc.IPPORT.FTP        IS 21:
VAL  soc.IPPORT.TELNET     IS 23:
VAL  soc.IPPORT.SMTP       IS 25:
VAL  soc.IPPORT.TIMESERVER IS 37:
VAL  soc.IPPORT.NAMESERVER IS 42:
VAL  soc.IPPORT.WHOIS      IS 43:
VAL  soc.IPPORT.MTP        IS 57:

--
-- Port/socket numbers: host specific functions
--
VAL  soc.IPPORT.TFTP       IS 69:
VAL  soc.IPPORT.RJE        IS 77:
VAL  soc.IPPORT.FINGER     IS 79:
VAL  soc.IPPORT.TTYLINK    IS 87:
VAL  soc.IPPORT.SUPDUP     IS 95:

--
-- UNIX TCP sockets
--
VAL  soc.IPPORT.EXECSERVER IS 512:
VAL  soc.IPPORT.LOGINSERVER IS 513:
VAL  soc.IPPORT.CMDSERVER  IS 514:
VAL  soc.IPPORT.EFSSERVER  IS 520:

--
-- UNIX UDP sockets
--
VAL  soc.IPPORT.BIFFUDP    IS 512:
VAL  soc.IPPORT.WHOSERVER  IS 513:
VAL  soc.IPPORT.ROUTESERVER IS 520:        -- 520+1 also used

--
-- Ports < IPPORT.RESERVED are reserved for
-- privileged processes (e.g. root).
-- Ports > IPPORT.USERRESERVED are reserved
-- for servers, not necessarily privileged.
--
VAL  soc.IPPORT.RESERVED   IS 1024:
VAL  soc.IPPORT.USERRESERVED IS 5000:

--
-- Link numbers
--
VAL  soc.IMPLINK.IP        IS 155:
VAL  soc.IMPLINK.LOWEXPER  IS 156:
VAL  soc.IMPLINK.HIGHEXPER IS 158:

--
-- Definitions of bits in internet address integers.
-- On subnets, the decomposition of addresses to host and net parts
-- is done according to subnet mask, not the masks here.
--
VAL  soc.IN.CLASSA.NET     IS #FF000000 (INT32):
VAL  soc.IN.CLASSA.NSHIFT  IS 24:
VAL  soc.IN.CLASSA.HOST    IS #00FFFFFF (INT32):
VAL  soc.IN.CLASSA.MAX     IS 128       (INT32):

VAL  soc.IN.CLASSB.NET     IS #FFFF0000 (INT32):
VAL  soc.IN.CLASSB.NSHIFT  IS 16:
VAL  soc.IN.CLASSB.HOST    IS #0000FFFF (INT32):
VAL  soc.IN.CLASSB.MAX     IS 65536     (INT32):

VAL  soc.IN.CLASSC.NET     IS #FFFFFF00 (INT32):
VAL  soc.IN.CLASSC.NSHIFT  IS 8:
VAL  soc.IN.CLASSC.HOST    IS #000000FF (INT32):

VAL  soc.INADDR.ANY        IS #00000000 (INT32):
VAL  soc.INADDR.LOOPBACK   IS #7F000001 (INT32):
VAL  soc.INADDR.BROADCAST  IS #FFFFFFFF (INT32):        -- must be masked

VAL  soc.IN.LOOPBACKNET    IS 127:      -- official!

--
-- Options for use with [gs]etsockopt at the TCP level.
--
VAL  soc.TCP.NODELAY       IS #0001:
VAL  soc.TCP.MAXSEG        IS #0002:

--
-- Options for use with [gs]etsockopt at the IP level.
--
VAL  soc.IP.OPTIONS        IS 1:        -- set/get IP per-packet options

---------------------------------------------------------------------------

VAL  soc.IPVERSION              IS 4:

VAL  soc.IPOPT.CONTROL          IS #00:
VAL  soc.IPOPT.RESERVED1        IS #20:
VAL  soc.IPOPT.DEBMEAS          IS #40:
VAL  soc.IPOPT.RESERVED2        IS #60:

VAL  soc.IPOPT.EOL              IS 0:   -- end of option list
VAL  soc.IPOPT.NOP              IS 1:   -- no operation

VAL  soc.IPOPT.RR               IS 7:   -- record packet route
VAL  soc.IPOPT.TS               IS 68:  -- timestamp
VAL  soc.IPOPT.SECURITY         IS 130: -- provide s,c,h,tcc
VAL  soc.IPOPT.LSRR             IS 131: -- loose source route
VAL  soc.IPOPT.SATID            IS 136: -- satnet id (now obsolete)
VAL  soc.IPOPT.SSRR             IS 137: -- strict source route

--
-- Offsets to fields in options other than EOL and NOP.
--
VAL  soc.IPOPT.OPTVAL           IS 0:   -- option ID
VAL  soc.IPOPT.OLEN             IS 1:   -- option length
VAL  soc.IPOPT.OFFSET           IS 2:   -- offset within option
VAL  soc.IPOPT.MINOFF           IS 4:   -- min value of above

-- bits for security (not byte swapped)
VAL  soc.IPOPT.SECUR.UNCLASS    IS #0000:
VAL  soc.IPOPT.SECUR.CONFID     IS #F135:
VAL  soc.IPOPT.SECUR.EFTO       IS #789A:
VAL  soc.IPOPT.SECUR.MMMM       IS #BC4D:
VAL  soc.IPOPT.SECUR.RESTR      IS #AF13:
VAL  soc.IPOPT.SECUR.SECRET     IS #D788:
VAL  soc.IPOPT.SECUR.TOPSECRET  IS #6BC5:

VAL  soc.IP.MSS                 IS 576: -- default maximum segment size

VAL  soc.MAX.IPOPTLEN           IS 40:  -- max number of IP option bytes

---------------------------------------------------------------------------

--
-- Definition of type and code field values for ICMP packets.
--
VAL  soc.ICMP.ECHOREPLY         IS 0:   -- echo reply
VAL  soc.ICMP.UNREACH           IS 3:   -- dest unreachable, codes:
VAL  soc.ICMP.SOURCEQUENCH      IS 4:   -- packet lost, slow down
VAL  soc.ICMP.REDIRECT          IS 5:   -- shorter route, codes:
VAL  soc.ICMP.ECHO              IS 8:   -- echo service
VAL  soc.ICMP.TIMXCEED          IS 11:  -- time exceeded, code:
VAL  soc.ICMP.PARAMPROB         IS 12:  -- ip header bad
VAL  soc.ICMP.TSTAMP            IS 13:  -- timestamp request
VAL  soc.ICMP.TSTAMPREPLY       IS 14:  -- timestamp reply
VAL  soc.ICMP.IREQ              IS 15:  -- information request
VAL  soc.ICMP.IREQREPLY         IS 16:  -- information reply
VAL  soc.ICMP.MASKREQ           IS 17:  -- address mask request
VAL  soc.ICMP.MASKREPLY         IS 18:  -- address mask reply
 
VAL  soc.ICMP.MAXTYPE           IS 18:

VAL  soc.ICMP.UNREACH.NET       IS 0:   -- bad net
VAL  soc.ICMP.UNREACH.HOST      IS 1:   -- bad host
VAL  soc.ICMP.UNREACH.PROTOCOL  IS 2:   -- bad protocol
VAL  soc.ICMP.UNREACH.PORT      IS 3:   -- bad port
VAL  soc.ICMP.UNREACH.NEEDFRAG  IS 4:   -- IP.DF caused drop
VAL  soc.ICMP.UNREACH.SRCFAIL   IS 5:   -- src route failed

VAL  soc.ICMP.REDIRECT.NET      IS 0:   -- for network
VAL  soc.ICMP.REDIRECT.HOST     IS 1:   -- for host
VAL  soc.ICMP.REDIRECT.TOSNET   IS 2:   -- for tos and net
VAL  soc.ICMP.REDIRECT.TOSHOST  IS 3:   -- for tos and host

VAL  soc.ICMP.TIMXCEED.INTRANS  IS 0:   -- ttl==0 in transit
VAL  soc.ICMP.TIMXCEED.REASS    IS 1:   -- ttl==0 in reass

---------------------------------------------------------------------------

VAL  soc.FIONBIO                IS 1:
VAL  soc.SIOCATMARK             IS 2:
VAL  soc.SIOCURGRTN             IS 3:
VAL  soc.TCFLSH                 IS 4:
VAL  soc.FIONREAD               IS 5:
