/*
 *     Program:       itconfig
 *     Module:        %M%  %W% %G%
 *
 *     Copyright Inmos Limited 1990.
 *
 *     Purpose:       help and quit routines
 *                    for remote configuration program for TCP/IP transputer
 *                    LAN interface products.
 *
 *     Compatability: BSD UNIX (host) and Inmos 1.0 Beta UDP/IP (target node)
 *                    VAX/VMS running WIN/TCP
 *                    PC DOS running PCNFS
 *                    PC DOS running PC/TCP
 *
 *     History
 *        GJT     01/05/90      a start!
 *        GJT     16/05/90      updates for latest spec
 *        GJT     06/11/90      updates for edit router help
 *        GJT     15/02/91      Changed all occurences of EEPROM to NVRAM
 */


#include <stdio.h>
#include "itconfig.h" /**/
#include "types.h" /**/

extern BOOL edited_config;  /* Set to TRUE if parameters are changed */




void call_help ()
{

  fprintf(stderr, "%s\n", version);
  fprintf(stderr, "%s\n\n", copyright);
  fprintf(stderr, "Usage: itconfig EthernetAddress [NetworkName]\n");
  fprintf(stderr, "Where:\n");
  fprintf(stderr, "      EthernetAddress = xx:xx:xx:xx:xx:xx  (6 colon separated hex numbers)\n");
  fprintf(stderr, "      NetworkName     = name of network    (default is \"localnet\")\n");

  return;
}


void top_help ()
{
  printf("The following commands are available:\n");
  printf("E  edit network parameters          L  load network parameters\n");
  printf("K  load resource parameters         N  display network parameters\n");
  printf("R  display resource parameters      S  save network parameters\n");
  printf("V  verify configuration parameters  W  write configuration parameters\n");
  printf("Q  quit                             ?  display this help information\n\n");
  printf("When prompted for a reply values in square brackets give options\n");
  printf("                          values in round brackets gives default\n");
  printf("To get default press RETURN\n");
  printf("If no default is offered RETURN normally quits the command\n\n");



  return;
}


void load_help ()
{
  printf("The following load data commands are available:\n");
  printf("F  load from file      N  load from NVRAM\n");
  printf("R  load from RAM       Q  quit this command\n\n");
  printf("?  display this help information\n\n");

  return;
}


void display_help ()
{
  printf("The following display data commands are available:\n");
  printf("C  display current configuration  F  display from file\n");
  printf("N  display from NVRAM             R  display from RAM\n");
  printf("Q  quit this command              ?  display this help information\n\n");

  return;
}


void router_edit_command_help ()
{
  printf("The following commands are available:\n");
  printf("E  edit this entry           D         delete this entry\n");
  printf("G  go to given entry         I         insert (before) entry\n");
  printf("N  insert (after) entry      P         move to previous entry\n");
  printf("Q  quit editing              [RETURN]  move to next entry\n");
  printf("?  display this help information\n");

  printf("Delete, edit and go to are not available if there are no entries\n");

  return;
}

void type_help ()
{
  printf("The following commands are available:\n");
  printf("D  default router entry   H  Host router entry\n");
  printf("N  network router entry   Q  Quit editing\n");
  printf("?  display this help information\n");

  printf("Quit is NOT available if editing an existing entry\n");

  return;
}


BOOL quit ()
{
  BOOL do_quit = TRUE;

  if (edited_config)
  {
    char reply[COMMAND_LEN];

    printf("Configuration has changed, do you still want to quit (N) [Y(es), N(o)]: ");
    fgets(reply, sizeof(reply), stdin);
    putchar('\n');


    if ((reply[0] != 'Y') && (reply[0] != 'y'))
        do_quit = FALSE;

  }

  return(do_quit);
}


