/*{{{  Header*/
/*
 *     Program:       itconfig
 *     Module:        %M%   %W% %G%
 *
 *
 *     Copyright Inmos Limited 1990.
 *
 *     Purpose:       main body of remote configuration program for TCP/IP
 *                    transputer LAN interface products.
 *
 *     Compatability: BSD UNIX (host) and Inmos 1.0 Beta UDP/IP (target node)
 *                    VAX/VMS running WIN/TCP
 *                    PC NFS
 *                    PC TCP
 *
 *     History
 *        GJT     01/05/90      a start!
 *        GJT     16/05/90      updates for latest spec
 *        GJT     12/02/91      now detects bad format Ethernet address
 *        GJT     15/02/91      Changed all occurences of EEPROM to NVRAM
 *        DAE     28/06/91      Added DEC/TCP support
 *        NMGS    17/01/92      Added PC/TCP support
 *        NMGS    23/03/92      Upgraded to PC-NFS toolkit v2.0 from v1.0
 */


/*}}}  */

/*{{{  includes and definitions*/

/*
 *         INCLUDE Files
 */


#include <stdio.h>
#include <errno.h>

#ifdef DECTCP
#include <netdb.h>
#include <types.h>
#include <socket.h>
#include <in.h>
#else
#include <netdb.h>
#ifdef PCNFS
#include <types.h>
#include <sys/tk_types.h>
#include <netinet/in.h>
#include <in_addr.h>
#else
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif /* PCNFS */
#include <sys/socket.h>
#ifdef VMS
#include <string.h>
#endif /* VMS */
#endif /* DECTCP */

#ifdef WINTCP
#include <ssdef.h>
#include <psldef.h>
#include <iodef.h>
#include <string.h>
#endif

#include "types.h" /**/
#include "itconfig.h" /**/

#ifdef WINTCP
extern int uerrno;
#define ERRNO uerrno
#endif

#ifdef DECTCP
#define ERRNO errno
#endif

#ifdef PCNFS
/* extern int errno; */
#define ERRNO errno
#endif

#ifdef PCTCP
#define ERRNO errno
#endif

#ifdef SUNTCP
#define ERRNO errno
#endif

extern char * getenv ();

/*
 * Some socket definitions
 */

#define REMOTE_CONFIG_PORT (5124)

#ifdef PCNFS
/*
 *INADRR_BROADCAST
 */
#define INADDR_BROADCAST  0XFFFFFFFF
#endif

/*}}}  */

/*{{{  external routines*/

/*
 * External function defs
 */

extern void call_help ();
extern void top_help ();
extern BOOL quit ();

#ifdef VMS
static void usleep();
#endif
int timed_read();
extern int set_options ();
/*}}}  */

/*{{{  global variables*/
/*****************************************************************************/
/*
 * Global variables
 */

unsigned int ether_addr[6];  /* Ethernet address of transputer TCP/IP system */

char id = 0;  /* Id for comms with transputer TCP/IP system */

unsigned char current_config_net[NET_DATA_SIZE];
unsigned char current_config_resource[RESOURCE_DATA_BLOCK];
int current_net_len = 0;
int current_resource_len = 0;
int SPEED_TABLE[NO_SPEEDS] = {20, 10, 5};

BOOL edited_config = FALSE;  /* Set to TRUE if parameters are changed */

int skt;                    /* socket id for comms with transputer TCP/IP system */
struct sockaddr_in skt_in, rsin;
#ifdef PCTCP
/* PC TCP does not support the netent structure
 * so we manufacture the relevant part
 */
struct { long n_net; } *net;
#else
struct netent *net;
#endif
char saved_net_filename[MAX_NAME_LEN];

/*****************************************************************************/
/*}}}  */

/*{{{  debug*/
#define debugging TRUE
void debug_m (n1, n2, string)
int n1, n2;
char *string;
{
  if (debugging)

    printf("DEBUG (%d, %d): %s\n", n1, n2, string);

  return;
}
/*}}}  */

/*{{{  run_configuration*/
static void run_configuration ()
/*
 * Top level command interface
 */
{
  char command[COMMAND_LEN];
  BOOL running = TRUE;

  while (running == TRUE)
  {
    printf("\nEnter command [E(dit), L(oad network), K(load resource), N(display network),\n");
    printf("               R(display resource), S(ave), V(erify), W(rite), Q(uit), ?]:  ");
    fgets(command, sizeof(command), stdin);
    putchar('\n');
    switch (command[0])
    {
      case 'E': case 'e':
        edit_network();
        break;
      case 'L': case 'l':
        load_network();
        break;
      case 'K': case 'k':
        load_resource();
        break;
      case 'N': case 'n':
        display_network();
        break;
      case 'R': case 'r':
        display_resource();
        break;
      case 'S': case 's':
        save_network();
        break;
      case 'V': case 'v':
        verify_configuration();
        break;
      case 'W': case 'w':
        write_configuration();
        break;
      case '?':
        top_help();
        break;
      case 'Q': case 'q':
        running = ! quit();
        break;
      case '\n':
        break;
      default:
        top_help();
        break;
    }  /* end of switch */
  }

  return;

}

/*}}}  */

/*{{{  main*/
main (argc, argv)
int argc;
char *argv[];

{
  char dummy;

  /*{{{  bad number of arguments  -- give help*/
  if ((argc != 2) && (argc != 3)) 
  {
    call_help();    /* Bad number of arguments */
    exit(OK_EXIT);
  }
  else if (sscanf (argv[1], "%x:%x:%x:%x:%x:%x%c", &ether_addr[0], &ether_addr[1],
              &ether_addr[2], &ether_addr[3], &ether_addr[4], &ether_addr[5], &dummy) != 6)
  {
    call_help();    /* Bad address format */
    fprintf (stderr, "\nError: bad format Ethernet address\n");
    exit(OK_EXIT);
  }
  /*}}}  */
  /*{{{  right number of arguments*/
  else  /* Number of arguments is ok */
  {
    char network_name[MAX_NAME_LEN];
    char *environment_ptr;
  
    printf("%s\n\n", version);
  
    if (argc == 3) /* Get netywork name from command line or set to default */
      strcpy(network_name, argv[2]);
    else
      strcpy(network_name, DEFAULT_NETWORK);
  
    saved_net_filename[0] = '\0';
  
    /*
     * Internal error detection - assert should never fail or its a programming error
     */
  
    if ((NET_DATA_SIZE + RESOURCE_DATA_SIZE + RESERVED_SIZE) != BLOCK_SIZE)
    {
      printf("Internal error: block size inconsistent - please report\n");
      exit(ERROR_EXIT);
    }
  
    /*
     * Open the connection
     */
  
    /*{{{  open the connection*/
    skt = socket (PF_INET, SOCK_DGRAM, 0);
    if (skt < 0)
    {
      perror ("socket");
      exit(ERROR_EXIT);
    }
    
    if (set_options(skt) < 0)
    {
      if (ERRNO == EPERM) {
        printf("\nError: You need to have root permission to use this program\n");
        exit(ERROR_EXIT);
      } else {
        perror ("setsockopt");
        exit(ERROR_EXIT);
      }
    }
    
    /*}}}  */
  /*}}}  */
#ifdef PCTCP
    /* PC TCP does not support the getnetbyname suite of function calls.
     * Therefore, broadcast to the 'local network' address.
     */

    net->n_net = htonl(INADDR_BROADCAST);
#else
    /*
     * Find the ip address of the local network
     */
    
    net = getnetbyname(network_name);
    if (net ==  NULL)
    {  /* Name not found, report and terminate */
      fprintf (stderr, "Error: network name %s not defined", network_name);
      perror ("");
      fprintf (stderr, "    The network %s must be defined in your networks database\n",
               network_name);
      exit(ERROR_EXIT);
    }
    
    if (net->n_net != 0)
    {    /* This replaces a call to inet_makeaddr, which  does not */
         /* appear to work on PCNFS */
    
      while ((net->n_net & 0XFF000000) == 0)
        net->n_net = ((net->n_net) << 8) | (INADDR_BROADCAST & 0xff);
      net->n_net = htonl(net->n_net);
    
    }
#endif

    /* Bind to receive any incoming dgrams for the allocated port */
    
    skt_in.sin_family = AF_INET;
    skt_in.sin_addr.s_addr = htonl (INADDR_ANY);
    skt_in.sin_port = htons (0);
    
    if (bind (skt, (struct sockaddr *) & skt_in, sizeof (skt_in)) < 0) {
        perror ("bind");
        exit(ERROR_EXIT);
    }
    
    /*
     * Specify the target address/port as broadcast/config_port
     */
    
    rsin.sin_family = AF_INET;
    rsin.sin_addr = *(struct in_addr *)(&net->n_net);
    rsin.sin_port = htons (REMOTE_CONFIG_PORT);
    
/*}}}  */
  
    /*
     * Read the initial configuration
     */
  
    current_net_len = read_net_NVRAM(current_config_net);
    if (current_net_len < NET_DATA_SIZE)
      printf("Warning: could not read network data\n");
    current_resource_len = read_resource_NVRAM(current_config_resource); 
    if (current_resource_len < RESOURCE_DATA_BLOCK)
      printf("Warning: could not read resource data\n");
  
    environment_ptr = (char *) getenv(RESOURCE_ENVIRONMENT);
    if (environment_ptr == NULL)
    {
      printf("Warning: %s %s not defined\n", environment_msg, RESOURCE_ENVIRONMENT);
    }
  
    run_configuration();
  
  }

}

