/*
 *     Program:       itconfig
 *     Module:        %M%   %W% %G%
 *
 *
 *     Copyright Inmos Limited 1990, 1991.
 *
 *     Purpose:       header definitions and constants for remote configuration
 *                    program for TCP/IP transputer LAN interface products.
 *
 *     Compatability: BSD UNIX (host) and Inmos 1.0 Beta UDP/IP (target node)
 *                    VAX/VMS running WIN/TCP or DEC/TCP
 *                    PC NFS
 *                    PC TCP
 *
 *     History
 *        GJT     01/05/90      a start!
 *        GJT     15/05/90      updates for latest spec
 *        GJT     31/10/90      updates for alpha release
 *        GJT     11/02/91      added constants for version numbers
 *        GJT     18/03/91      Changes for new editing mechanism
 *        GJT     05/04/91      Added default host and domain names, moved to 1.23
 *        DAE     28/06/91      Added DEC/TCP support
 *        NMGS    17/01/92      Added PC/TCP support
 */

/* 
 * Assert that a name has been defined
 */

#ifndef SUN3
#ifndef SUN4
#ifndef PC
#ifndef VAX
error no valid host machine specified
#endif
#endif
#endif
#endif

#ifndef SUNOS
#ifndef VMS
#ifndef DOS
error no valid operating system specified
#endif
#endif
#endif

#ifndef SUNTCP
#ifndef WINTCP
#ifndef DECTCP
#ifndef PCNFS
#ifndef PCTCP
error no valid host network specified
#endif
#endif
#endif
#endif
#endif

/*
 * IFDEF'd values
 */

#ifdef VMS
#define OK_EXIT     (1)
#define ERROR_EXIT (42)
#else
#define OK_EXIT     (0)
#define ERROR_EXIT  (1)
#endif


/*
 * Strings and string related constants
 */

#define version "itconfig: transputer Ethernet interface configurer, V 1.32"
#define copyright "(c) Copyright INMOS Limited 1990, 1991"
#define verify_string "inmos.com(tcplink-config-V01.00)"
#define netfile_version "TCPlink network parameter file, Version 01.21"

#ifdef VMS
#define environment_msg "logical name"
#else
#define environment_msg "environment variable"
#endif


#define DEFAULT_HOST_NAME "???"
#define DEFAULT_DOMAIN_NAME "."

#define DEFAULT_TYPE_S "DEFAULT"
#define NETWORK_TYPE_S "NETWORK"
#define HOST_TYPE_S    "HOST"

#define NO_TOS_S    "TOS unspecified"
#define TOS_S       "TOS specified"

/* 
 * Must be able to hold maximum of  and TOS
 * strings above
 */

#define MAX_TYPE_SLEN (30) 

/*
 * NETFILE_LIMIT IS How many chars to compare of version string above
 * value <= 40 means that version number is not checked
 */

#define NETFILE_LIMIT  (45)

/* RESOURCE_ENVIRONMENT is environment variable, logical name etc.,
 * giving resource database file name.
 */

#define RESOURCE_ENVIRONMENT "ICONDB"

#define DEFAULT_NETWORK "localnet"

#define MAX_DOTBYTES   (10)
#define MAX_COLONBYTES  (4)
#define ETHER_ADDR_LEN  (6)
#define IP_ADDR_LEN     (4)

#define COMMAND_LEN     (82)
/* We only need 1 byte command char, 1 byte '\n', 1 byte '\0' */
/* But allow for an entire line plus NL and NULL */

#define MAX_ERRORS (10)

#define MAX_NET_LINE_SIZE (255)

/*
 * data block definitions
 */

#define BLOCK_SIZE           (4096)
#define MAX_ROUTER_ENTRIES    (128)
#define NET_DATA_SIZE        (1764)
#define RESOURCE_DATA_SIZE   (2144) 
#define RESERVED_SIZE         (188)
#define RESOURCE_DATA_BLOCK  (RESOURCE_DATA_SIZE + RESERVED_SIZE) 
#define MAJOR_OFFSET            (6) /* The offset of the major verion no */
#define MINOR_OFFSET            (7) /* The offset of the minor verion no */
#define CHANGE_OFFSET           (8) /* The offset to start of changeable data */
#define SYSTEM_NAME_OFFSET     (32) /* The offset of TCP/IP system name in block */
#define DOMAIN_NAME_OFFSET     (97) /* The offset of TCP/IP domain name in block */
#define ROUTER_OFFSET         (356) /* The offset of router table */
#define ROUTER_ENTRY_SIZE      (11)
#define MAX_NAME_LEN           (64)
#define MAX_DOMAIN_LEN        (255)
#define MAX_RESOURCE_ENTRIES   (32)
#define RESOURCE_ENTRY_SIZE  (MAX_NAME_LEN + 3)
#define RESOURCE_ALLOCATED_O    (1)
#define RESOURCE_LEN_O          (2)
#define RESOURCE_NAME_O         (3)


#define TRANSPUTER_FALSE     (0)


/* Number of different link speeds */

#define NO_SPEEDS (3)   


/*
 * Router table constants
 */

#define HOST_TYPE              (0)
#define NO_TOS_TYPE            (1)
#define NETWORK_TYPE           (2)
#define DEFAULT_TYPE           (8)
#define NOT_IN_USE_TYPE       (16)
#define END_OF_ROUTER_TABLE   (64)
#define VALID_ENTRY          (128)

/* Mask to find important parts of type field */
#define TYPE_MASK (255)

/* Mask to remove TOS not required field */
#define TOS_MASK  (254)

/*
 * Network configuration file constants
 */

/*
 * Line on which host name is specified
 */

#define NAME_LINE_NO (9)


/*
 * Request/Reply protocol and related constants
 */

#define MAX_RETRIES (10)

#define VFIELD_SIZE    (40)  /* Verification string field size */
#define MIN_FRAME_SIZE (52)  /* Shortest frame (no data) */
#define DATA_LEN       (512) /* Most data */
#define MAX_DATA_LEN   (516) /* Data plus 2 bytes address and 2 bytes length */
#define MAX_DATAGRAM_LEN (MIN_FRAME_SIZE + MAX_DATA_LEN)

#define EREAD_REQ   (0)
#define MREAD_REQ   (1)
#define WRITE_REQ   (2)

#define EREAD_REPLY (3)
#define MREAD_REPLY (4)
#define WRITE_REPLY (5)

#define NOP (99)



/* 
 * Defines range of link values permitted in connection database file
 * a value of 4 means range 0 - 3 is valid.
 */
#define MAX_SUBSYSTEMS (4)


/*
 * Unsigned int version of router table for convenience
 */
struct router_entry {
		     struct router_entry *prev;
		     unsigned int type;
		     unsigned int address[IP_ADDR_LEN];
		     unsigned int TOS;
		     unsigned int gateway[IP_ADDR_LEN];
		     struct router_entry *next;
		    };



