/*{{{  Header*/
/*
 *     Program:       itconfig
 *     Module:        %M%  %W% %G%
 *
 *
 *     Copyright Inmos Limited 1990.
 *
 *     Purpose:       low level and VAX/VMS specific routines
 *                    for remote configuration program for TCP/IP transputer
 *                    LAN interface products.
 *
 *     Compatability: BSD UNIX (host) and Inmos 1.0 Beta UDP/IP (target node)
 *                    VAX/VMS running WIN/TCP
 *                    PC NFS
 *
 *     History
 *        GJT     01/05/90      a start!
 *        GJT     18/05/90      mods for PCNFS port
 *        GJT     15/02/91      Changed all occurences of EEPROM to NVRAM
 *        DAE     28/06/91      Added DEC/TCP support
 *        NMGS    22/03/92      Upgraded PC-NFS toolkit support to v2.0
 */


/*}}}  */

#include <stdio.h>

#ifdef DECTCP
#include <time.h>
#include <types.h>
#include <socket.h>
#include <in.h>
#include <inet.h>
#else
#ifdef PCNFS
#include <sys/tk_types.h>
#include <sys/nfs_time.h>
#include <netinet/in.h>
#include <in_addr.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif /* PCNFS */
#include <sys/socket.h>
#endif /* DECTCP */

#include "itconfig.h" /**/
#ifdef PCNFS
/* #include "pcntypes.h" Now supplied in tk_types.h */
#endif

extern int skt;
extern struct sockaddr_in skt_in, rsin;
extern struct netent *net;

extern void debug_m();

#define fd_limit (FD_SETSIZE)

/*{{{  timed_read*/
int timed_read (s, buf, len, millisec)
int s;
unsigned char *buf;
int len;
long int millisec;
/*
 * Returns the number of bytes successfully read or zero if
 * timeout after time in 'ms' milliseconds
 */
{
    int fdwidth, result, len_read;
    struct timeval timeout;

#ifdef DECTCP
    int readfds;
#else /* DECTCP */
    fd_set readfds;
#endif /* DECTCP */

    timeout.tv_sec = millisec / 1000;
    timeout.tv_usec = (millisec % 1000) * 1000;

#ifdef DECTCP
    readfds = (1 << s);
#else
    FD_ZERO (&readfds);
    FD_SET (s, &readfds);
#endif

    fdwidth = 32;

#ifdef DECTCP
    result = select(fdwidth, &readfds, (int *) NULL, (int *) NULL, &timeout);
#else
    result = select(fdwidth, &readfds, (fd_set *) NULL, (fd_set *) NULL, &timeout);
#endif /* DECTCP */

    if (result > 0)
    {
      int addr_len = sizeof(struct sockaddr);

      /* len_read = recv(s, (char *) buf, len, 0); */
      len_read = recvfrom(s, (char *) buf, len, 0,
		     (struct sockaddr *) &skt_in, &addr_len);
      return(len_read);
    }
    else
    {
      return (0);
    }
}
/*}}}  */

#ifdef PCNFS
/*{{{  set_options*/
int set_options (skt)
int skt;
/*
 * Dummy set socket options for PCNFS which does not allow setting of broadcast
 * options (hopefully it allows it anyway)
 */
{
  return(0);
}
/*}}}  */
#else
/*{{{  set_options*/
int set_options (skt)
int skt;
/*
 * Set socket options for non-PCNFS system to allow broadcast datagrams.
 */
{
  int result;
  int on = 1;

  result = setsockopt (skt, SOL_SOCKET, SO_BROADCAST, &on, sizeof (on));

  return(result);
}
/*}}}  */
#endif
