#ifdef VAX
/*
 * Copyright (c) 1989 by Digital Equipment Corporation, Maynard, Mass.
 *
 * This software is furnished under a license and may be used and copied only
 * in accordance with the terms of such license and with the inclusion of the
 * above copyright notice. This software or any other copies thereof may not
 * be provided or otherwise made available to any other person. No title to
 * and ownership of the software is hereby transferred.
 *
 * The information in this software is subject to change without notice and
 * should not be construed as a commitment by Digital Equipment Corporation.
 *
 * Digital assumes no responsibility for the use or reliability of its
 * software on equipment which is not supplied by Digital.
 *
 * Abstract:
 *
 *	ioctl: implements the Unix ioctl call.
 *		This function is not intended to be a full ioctl implementation.
 *		Only the functions related to inet are of interest.
 *
 * Environment:
 *	UCX v1.0 or higher, VMS v4.7 or higher.
 *
 * Authors:
 *	UCX developer.
 *
 * Creation Date:
 *	23rd May, 1989.
 *
 */

/*
 * Include files
 */
#include <stdio.h>
#include <errno.h>
#include <iodef.h>
#include <ucx$inetdef.h>

/*
 * Functional description
 *
 * Ioctl's have the command encoded in the lower word, and the size of any
 * in or out parameters in the upper word. The high 2 bits of the upper word
 * are used to encode the in/out status of the parameter; for now we restrict
 * parameters to at most 128 bytes. The IOC_VOID field of 0x20000000 is defined
 * so that new ioctls can be distinguished from old ioctls.
 *
 * Formal parameters
 *	d	file or socket descriptor
 *	request	defined in ioctl.h
 *	argp	"in" or "out" parameter
 *
 * Routine value
 *	status code
 *
 */

/*
 * Macro definitions
 */

#ifndef _IO
#define IOCPARM_MASK	0x7f	/* Params are < 128 bytes */
#define IOC_VOID	(int)0x20000000	/* No params */
#define IOC_OUT		(int)0x40000000	/* Copy out params */
#define IOC_IN		(int)0x80000000	/* Copy in params */
#define IOC_INOUT	(int)(IOC_IN|IOC_OUT)
#define _IO(x,y)	(int)(IOC_VOID|('x'<<8)|y)
#define _IOR(x,y,t)	(int)(IOC_OUT|((sizeof(t)&IOCPARM_MASK)<<16)|('x'<<8)|y)
#define _IOW(x,y,t)	(int)(IOC_IN|((sizeof(t)&IOCPARM_MASK)<<16)|('x'<<8)|y)
#define _IOWR(x,y,t)	(int)(IOC_INOUT|((sizeof(t)&IOCPARM_MASK)<<16)|('x'<<8)|y)
#endif				/* _IO */

#define VMSOK(s)	((s) & 01)

int
ioctl (d, request, argp)
     int             d, request;
     char           *argp;
{
    int             ef;		/* Event flag number */
    int             sdc;	/* Socket device channel */
    unsigned short  fun;	/* Qiow function code */
    unsigned short  iosb[4];	/* Io status block */
    char           *p5, *p6;	/* Args p5 & p6 of qiow */

    struct comm
    {
	int             command;
	char           *addr;
    }
    ioctl_comm;			/* Qiow ioctl commands */

    struct it2
    {
	unsigned short  len;
	unsigned short  opt;
	struct comm    *addr;
    } ioctl_desc;		/* Qiow ioctl commands descriptor */

    int             status;

    /*
     * Gets an event flag for qio.
     */
    status = LIB$GET_EF (&ef);
    if (!VMSOK (status))
    {
	/* No ef available. Use 0. */
	ef = 0;
    }

    /*
     * Get the socket device channel number.
     */
    sdc = vaxc$get_sdc (d);
    if (sdc == 0)
    {
	/* Not an open socket descriptor. */
	errno = EBADF;
	return (-1);
    }

    /*
     * Fill in ioctl descriptor.
     */
    ioctl_desc.opt = UCX$C_IOCTL;
    ioctl_desc.len = sizeof (struct comm);

    ioctl_desc.addr = &ioctl_comm;

    /*
     * Decide qio function code and in/out parameter.
     */
    if (request & IOC_OUT)
    {
	fun = IO$_SENSEMODE;
	p5 = 0;
	(struct it2 *) p6 = &ioctl_desc;
    }
    else
    {
	fun = IO$_SETMODE;
	(struct it2 *) p5 = &ioctl_desc;
	p6 = 0;
    }

    /*
     * Fill in ioctl command.
     */
    ioctl_comm.command = request;
    ioctl_comm.addr = argp;

    /*
     * Do ioctl.
     */
    status = SYS$QIOW (ef, sdc, fun, iosb, 0, 0,
		       0, 0, 0, 0,	/* p1 - p4: not used */
		       p5, p6);

    if (!VMSOK (status))
    {

#ifdef DEBUG
	fprintf (stderr, "ioctl failed: status = %d\n", status);
#endif				/* DEBUG */

	errno = status;
	return (-1);
    }

    if (!VMSOK (iosb[0]))
    {

#ifdef DEBUG
	fprintf (stderr, "ioctl failed: status = 0x%x, 0x%x, 0x%x%x\n",
		 iosb[0], iosb[1], iosb[3], iosb[2]);
#endif				/* DEBUG */

	errno = iosb[0];
	return (-1);
    }

    status = LIB$FREE_EF (&ef);

    return (0);
}

#endif				/* VAX */
